"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const pulumi_1 = require("@pulumi/pulumi");
const ec2_1 = require("@pulumi/aws/ec2");
const aws_1 = require("@pulumi/aws");
const rds_1 = require("@pulumi/aws/rds");
const elasticache_1 = require("@pulumi/aws/elasticache");
const VpcDns_1 = require("./VpcDns");
const VpcSubnetGroup_1 = require("./VpcSubnetGroup");
const VpcNat_1 = require("./VpcNat");
const VpcCidr_1 = require("./VpcCidr");
/**
 * A `VpcNetwork` provides multiple resources to establish a consistent baseline for logical
 * networking organization.
 */
class VpcNetwork extends pulumi_1.ComponentResource {
    constructor(name, options, extra) {
        var _a, _b;
        super('sophosoft:aws:VpcNetwork', name, {}, extra);
        /**
         * `RouteTables` for internal resources
         */
        this.internalRoutes = [];
        /**
         * `NatInstances` and their associated `RouteTables`
         * @see internalRoutes
         */
        this.nats = [];
        this.namePrefix = options.name.toLowerCase().replace('_', '-');
        this.vpc = new ec2_1.Vpc(`${name}-vpc`, {
            cidrBlock: options.cidrBlock,
            enableDnsHostnames: (options.dnsOptions !== null),
            enableDnsSupport: (options.dnsOptions !== null),
            tags: {
                Name: options.name
            }
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.gateway = new ec2_1.InternetGateway(`${name}-igw`, {
            vpcId: this.vpc.id,
            tags: {
                Name: `${this.namePrefix}-igw`
            }
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.externalRoutes = new ec2_1.RouteTable(`${name}-external`, {
            vpcId: this.vpc.id,
            routes: [{
                    cidrBlock: '0.0.0.0/0',
                    gatewayId: this.gateway.id
                }],
            tags: {
                Name: `${this.namePrefix}-external`
            }
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        if (options.dnsOptions) {
            this.dns = new VpcDns_1.VpcDns(`${name}-dns`, {
                domainName: options.dnsOptions.domain,
                nameServers: options.dnsOptions.nameServers,
                prefix: this.namePrefix,
                region: options.dnsOptions.region,
                vpcId: this.vpc.id,
            }, Object.assign(Object.assign({}, extra), { parent: this }));
        }
        const azList = aws_1.getAvailabilityZones({ state: 'available' }).names.slice(0, options.maxZones || 3);
        const vpcCidr = new VpcCidr_1.VpcCidr(options.cidrBlock, azList, (_a = options.subnetOptions) === null || _a === void 0 ? void 0 : _a.azAdditionalBits, (_b = options.subnetOptions) === null || _b === void 0 ? void 0 : _b.subnetAdditionalBits);
        this.externalSubnets = new VpcSubnetGroup_1.VpcSubnetGroup(`${name}-subnets-external`, {
            namePrefix: this.namePrefix,
            subnetCidrBlocks: vpcCidr.externalBlocks,
            routeTables: [this.externalRoutes],
            type: 'external',
            vpcId: this.vpc.id
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.externalSubnets.subnets.forEach((subnet, index) => {
            const nat = new VpcNat_1.VpcNat(`${name}-nat-${index + 1}`, {
                azLetter: azList[index][azList[index].length - 1],
                namePrefix: this.namePrefix,
                subnetId: subnet.subnet.id,
                vpcId: this.vpc.id
            }, Object.assign(Object.assign({}, extra), { parent: this }));
            this.nats.push(nat);
            this.internalRoutes.push(nat.routes);
        });
        this.internalSubnets = new VpcSubnetGroup_1.VpcSubnetGroup(`${name}-subnets-internal`, {
            namePrefix: this.namePrefix,
            subnetCidrBlocks: vpcCidr.internalBlocks,
            routeTables: this.internalRoutes,
            type: 'internal',
            vpcId: this.vpc.id
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.dataSubnets = new VpcSubnetGroup_1.VpcSubnetGroup(`${name}-subnets-data`, {
            namePrefix: this.namePrefix,
            subnetCidrBlocks: vpcCidr.dataBlocks,
            routeTables: this.internalRoutes,
            type: 'data',
            vpcId: this.vpc.id
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.rdsSubnetGroup = new rds_1.SubnetGroup(`${name}-subnetGroup-rds`, {
            name: `${this.namePrefix}-db-subnets`,
            description: `Data Subnets for ${options.name}`,
            subnetIds: this.dataSubnets.subnets.map((sn) => sn.subnet.id),
            tags: {
                Name: `${this.namePrefix}-db-subnets`
            }
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.ecSubnetGroup = new elasticache_1.SubnetGroup(`${name}-subnetGroup-ec`, {
            name: `${this.namePrefix}-ec-subnets`,
            description: `Data Subnets for ${options.name}`,
            subnetIds: this.dataSubnets.subnets.map((sn) => sn.subnet.id)
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.networkSubnets = new VpcSubnetGroup_1.VpcSubnetGroup(`${name}-subnets-network`, {
            namePrefix: this.namePrefix,
            subnetCidrBlocks: vpcCidr.networkBlocks,
            routeTables: this.internalRoutes,
            type: 'network',
            vpcId: this.vpc.id
        }, Object.assign(Object.assign({}, extra), { parent: this }));
        this.registerOutputs({
            vpc: this.vpc,
            gateway: this.gateway,
            externalRoutes: this.externalRoutes,
            internalRoutes: this.internalRoutes,
            dns: this.dns,
            nats: this.nats,
            externalSubnets: this.externalSubnets,
            internalSubnets: this.internalSubnets,
            dataSubnets: this.dataSubnets,
            networkSubnets: this.networkSubnets,
            rdsSubnetGroup: this.rdsSubnetGroup,
            ecSubnetGroup: this.ecSubnetGroup
        });
    }
}
exports.VpcNetwork = VpcNetwork;
